/**
 *############################################################################
 * A component of the Greenstone Librarian Interface, part of the Greenstone
 * digital library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * Author: Michael Dewsnip, NZDL Project, University of Waikato, NZ
 *
 * Copyright (C) 2004 New Zealand Digital Library Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *############################################################################
 */

package org.greenstone.gatherer.collection;


import java.io.*;
import javax.swing.*;
import org.greenstone.gatherer.cdm.CollectionDesignManager;
import org.greenstone.gatherer.file.FileNode;
import org.greenstone.gatherer.util.JarTools;
import java.util.Set;
import java.util.Iterator;
import java.nio.charset.Charset;


/** This class represents one node in the collection tree. */
public class CollectionTreeNode
    extends FileNode
{
    static final public ImageIcon GREEN_FILE_ICON = JarTools.getImage("greenfile.gif", true);
    
    /** Is this file a metadata database that is explodable with the explode_metadata_databases.pl script? */
    private boolean is_explodable = false;

    /** Keeps track of whether this file can be replaced with a Greenstone-generated html file.
     * To work with replace_srcdoc_with_html.pl */
    private boolean is_srcreplaceable = false;

    private String displayFileName = null;

    public CollectionTreeNode(File file)
    {
	super(file);

	this.is_explodable = CollectionDesignManager.plugin_manager.isFileExplodable(file);
	// To work with replace_srcdoc_with_html.pl
	this.is_srcreplaceable = CollectionDesignManager.plugin_manager.isFileSrcReplaceable(file);
    
	displayFileName = calcDisplayString();
    }

    /** Similar to calcDisplayString, but this version only checks whether the filename
     * String is UTF8. If so, it converts it into UTF8 and returns it. If the filename
     * is not UTF8, it is returned unchanged.
     */ 
    protected String quickCalcDisplayString() {
	String filename = super.toString();
	try{
	    String utf8filename = new String(filename.getBytes(), "UTF8");
	    if(utf8filename.indexOf('\ufffd') == -1) {
		return utf8filename;
	    } else { // contains the character indicating that it's invalid utf8
		// return the original string
		return filename;
	    }
	} catch(java.io.UnsupportedEncodingException e) {
	    return filename;
	}
    }

    /** This method returns a string representation of the filenodes in the Collection
     * Tree, that can then be displayed in the tree. 
     * We'll initially assume that the filenames are utf8 encoded and so convert the
     * filename into utf8 for proper presentation in the Collection tree pane.
     * If the filenames are not utf8, then the conversion would have introduced funny
     * characters. Therefore, when converting to utf8, if the converted filename 
     * contains the special character '\ufffd', then we know the conversion did not work
     * and we return the original string which may or may not be properly presented by
     * default.
     * See http://java.sun.com/j2se/1.4.2/docs/api/java/nio/charset/CharsetDecoder.html
     * which says "How a decoding error is handled depends upon the action requested for 
     * that type of error, which is described by an instance of the CodingErrorAction class.
     * The possible error actions are to ignore the erroneous input, report the error to
     * the invoker via the returned CoderResult object, or replace the erroneous input with
     * the current value of the replacement string. The replacement has the initial value 
     * "\uFFFD"; its value may be changed via the replaceWith method."
     * The following made me think that String(byte[], String charsetName) constructor may
     * use the replacement value \uFFFD.
     * http://www.experts-exchange.com/Programming/Programming_Languages/Java/Q_20512969.html
     * mentions the following which made me think of this: 
     * convertedStr = convertedStr.replace('\ufffd', ' ');
     */
    protected String calcDisplayString() {
	String filename = super.toString();

	try{
	    String[] charsets = {"UTF-8", "ISO-8859-1", "US-ASCII", "UTF-16BE", "UTF-16LE", "UTF-16"};
	    for(int i = 0; i < charsets.length; i++) {
		String charset = charsets[i];
		String convertedfilename = new String(filename.getBytes(), charset);
		if(convertedfilename.indexOf('\ufffd') != -1) {
		    // encountered the character that indicates when the conversion is invalid
		    convertedfilename = null;
		    continue; // try to use the next charset to encode the filename as
		} 
		else { // valid conversion
		    System.err.println("Filename was encoded in: " + charset);
		    return convertedfilename;
		}
	    }
	} catch(java.io.UnsupportedEncodingException e) {
	    return filename;
	}

	return filename; // attempted conversions all failed
    }


    public FileNode addChildNode(File file)
    {
	CollectionTreeNode child_node = new CollectionTreeNode(file);
	child_node.setModel(model);
	child_node.setParent(this);
	return child_node;
    }


    public boolean isExplodable()
    {
	return is_explodable;
    }

    // To work with replace_srcdoc_with_html.pl
    public boolean isSrcReplaceable()
    {
	return is_srcreplaceable;
    }

    /** This method returns a string representation of the filenodes in the Collection
     * Tree, which is what will be displayed in the tree. It tries to convert it into
     * some common encoding formats. Failing that, the unchanged filepath is returned.
     * @see displayString
     */ 
    public String toString() 
    {
	if(displayFileName == null) {    
	  displayFileName = calcDisplayString();
	}
	return displayFileName;
    }
}
